﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;
using SpiceLogic.HtmlEditorControl.Domain.BOs.Entities.SpellCheck;

namespace WindowsFormsApplication1
{
    public class CustomSpellCheckerEngine : ISpellCheckerEngine
    {
        private readonly HashSet<string> _userDictionary = new HashSet<string>();
        private string _fixedUserDictionaryPath;
        
        
        /// <summary>
        /// Initializes the specified dictionary path.
        /// </summary>
        /// <param name="dictionaryPath">The dictionary path.</param>
        /// <param name="affixPath">The affix path.</param>
        /// <param name="userDictionaryPath">The user dictionary path.</param>
        public void Initialize(string dictionaryPath, string affixPath, string userDictionaryPath)
        {
            initializeUserDictionary(userDictionaryPath);
        }

        /// <summary>
        /// Spells the specified word.
        /// </summary>
        /// <param name="word">The word.</param>
        /// <returns></returns>
        public bool Spell(string word)
        {
            // This demo logic checks if a word starts with a, then, consider the word wrong spelled.
            // Otherwise, the word is considered correctly spelled.

            if (string.IsNullOrEmpty(word))
                return true;

            if (! word.StartsWith("a"))
                return true;

            if (_userDictionary.Contains(word))
                return true;

            return false;
        }

        /// <summary>
        /// Suggests the specified word.
        /// </summary>
        /// <param name="word">The word.</param>
        /// <param name="max">The maximum.</param>
        /// <returns></returns>
        public IEnumerable<string> Suggest(string word, int? max = null)
        {
            // This demo logic checks if a word starts with a, then, suggest 2 words 
            // from an array. Following this logic, you can create your own implementation.

            if (!string.IsNullOrEmpty(word) && word.StartsWith("a"))
                return new[] { "Aesthetics007", "Apple008" };

            return new string[] { };
        }

        /// <summary>
        /// Adds to user dictionary.
        /// </summary>
        /// <param name="word">The word.</param>
        /// <exception cref="System.NotSupportedException"></exception>
        public void AddToUserDictionary(string word)
        {
            if (String.IsNullOrEmpty(word))
                return;

            try
            {
                //create user dictionary if it does not exist
                if (!File.Exists(_fixedUserDictionaryPath))
                {
                    string directoryName = Path.GetDirectoryName(_fixedUserDictionaryPath) ?? "";
                    if (!Directory.Exists(directoryName))
                    {
                        MessageBox.Show("User dictionary directory does not exist");
                        return;
                    }

                    using (File.Create(_fixedUserDictionaryPath))
                    {
                    }
                }

                //do not add word twice
                string[] userDictionaryWords = File.ReadAllLines(_fixedUserDictionaryPath);
                if (Array.Exists(userDictionaryWords, s => s == word))
                    return;

                using (StreamWriter writer = File.AppendText(_fixedUserDictionaryPath))
                    writer.WriteLine(word);

                if (!_userDictionary.Contains(word))
                    _userDictionary.Add(word);
            }
            catch (UnauthorizedAccessException)
            {
                MessageBox.Show(string.Format("File cannot be created/updated at this path '{0}' due to permission restriction", _fixedUserDictionaryPath));
            }
            catch (Exception)
            {
                MessageBox.Show(string.Format("An unexpected error occurred during creating/updating user dictionary at '{0}'", _fixedUserDictionaryPath));
            }
        }

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        public void Dispose()
        {
            //not required
        }


        /// <summary>
        /// Fixes the dictionary path.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        /// <returns></returns>
        private static string fixDictionaryPath(string filePath)
        {
            if (String.IsNullOrEmpty(filePath))
                return filePath;

            if (!Path.IsPathRooted(filePath))
                filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, filePath);

            //add .dic extension if file does not exist and dictionary path does not have required extension
            if (!File.Exists(filePath) && !string.Equals(Path.GetExtension(filePath), ".dic", StringComparison.OrdinalIgnoreCase))
                filePath = filePath.Trim() + ".dic";

            return filePath;
        }

        /// <summary>
        /// Initializes the user dictionary.
        /// </summary>
        /// <param name="userDictionaryPath">The user dictionary path.</param>
        private void initializeUserDictionary(string userDictionaryPath)
        {
            _fixedUserDictionaryPath = fixDictionaryPath(userDictionaryPath);
            if (!String.IsNullOrEmpty(_fixedUserDictionaryPath) && File.Exists(_fixedUserDictionaryPath))
            {
                // Append to user dictionary all the words from specified file
                foreach (string line in File.ReadAllLines(_fixedUserDictionaryPath))
                {
                    if (!_userDictionary.Contains(line))
                        _userDictionary.Add(line);
                }
            }
        }
    }
}