﻿Imports System.IO
Imports SpiceLogic.HtmlEditorControl.Domain.BOs.Entities.SpellCheck

Public Class CustomSpellCheckerEngine
    Implements ISpellCheckerEngine
    Private ReadOnly _userDictionary As New HashSet(Of String)()
    Private _fixedUserDictionaryPath As String


    ''' <summary>
    ''' Initializes the specified dictionary path.
    ''' </summary>
    ''' <param name="dictionaryPath">The dictionary path.</param>
    ''' <param name="affixPath">The affix path.</param>
    ''' <param name="userDictionaryPath">The user dictionary path.</param>
    Public Sub Initialize(dictionaryPath As String, affixPath As String, userDictionaryPath As String) Implements ISpellCheckerEngine.Initialize
        initializeUserDictionary(userDictionaryPath)
    End Sub

    ''' <summary>
    ''' Spells the specified word.
    ''' </summary>
    ''' <param name="word">The word.</param>
    ''' <returns></returns>
    Public Function Spell(word As String) As Boolean Implements ISpellCheckerEngine.Spell
        ' This demo logic checks if a word starts with a, then, consider the word wrong spelled.
        ' Otherwise, the word is considered correctly spelled.

        If String.IsNullOrEmpty(word) Then
            Return True
        End If

        If Not word.StartsWith("a") Then
            Return True
        End If

        If _userDictionary.Contains(word) Then
            Return True
        End If

        Return False
    End Function

    ''' <summary>
    ''' Suggests the specified word.
    ''' </summary>
    ''' <param name="word">The word.</param>
    ''' <param name="max">The maximum.</param>
    ''' <returns></returns>
    Public Function Suggest(word As String, Optional max As System.Nullable(Of Integer) = Nothing) As IEnumerable(Of String) Implements ISpellCheckerEngine.Suggest
        ' This demo logic checks if a word starts with a, then, suggest 2 words 
        ' from an array. Following this logic, you can create your own implementation.

        If Not String.IsNullOrEmpty(word) AndAlso word.StartsWith("a") Then
            Return New String() {"Aesthetics007", "Apple008"}
        End If

        Return New String() {}
    End Function

    ''' <summary>
    ''' Adds to user dictionary.
    ''' </summary>
    ''' <param name="word">The word.</param>
    ''' <exception cref="System.NotSupportedException"></exception>
    Public Sub AddToUserDictionary(word As String) Implements ISpellCheckerEngine.AddToUserDictionary
        If String.IsNullOrEmpty(word) Then
            Return
        End If

        Try
            'create user dictionary if it does not exist
            If Not File.Exists(_fixedUserDictionaryPath) Then
                Dim directoryName As String = If(Path.GetDirectoryName(_fixedUserDictionaryPath), "")
                If Not Directory.Exists(directoryName) Then
                    MessageBox.Show("User dictionary directory does not exist")
                    Return
                End If

                Using File.Create(_fixedUserDictionaryPath)
                End Using
            End If

            'do not add word twice
            Dim userDictionaryWords As String() = File.ReadAllLines(_fixedUserDictionaryPath)
            If Array.Exists(userDictionaryWords, Function(s) s = word) Then
                Return
            End If

            Using writer As StreamWriter = File.AppendText(_fixedUserDictionaryPath)
                writer.WriteLine(word)
            End Using

            If Not _userDictionary.Contains(word) Then
                _userDictionary.Add(word)
            End If
        Catch generatedExceptionName As UnauthorizedAccessException
            MessageBox.Show(String.Format("File cannot be created/updated at this path '{0}' due to permission restriction", _fixedUserDictionaryPath))
        Catch generatedExceptionName As Exception
            MessageBox.Show(String.Format("An unexpected error occurred during creating/updating user dictionary at '{0}'", _fixedUserDictionaryPath))
        End Try
    End Sub

    ''' <summary>
    ''' Releases unmanaged and - optionally - managed resources.
    ''' </summary>
    Public Sub Dispose() Implements ISpellCheckerEngine.Dispose
        'not required
    End Sub


    ''' <summary>
    ''' Fixes the dictionary path.
    ''' </summary>
    ''' <param name="filePath">The file path.</param>
    ''' <returns></returns>
    Private Shared Function fixDictionaryPath(filePath As String) As String
        If String.IsNullOrEmpty(filePath) Then
            Return filePath
        End If

        If Not Path.IsPathRooted(filePath) Then
            filePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, filePath)
        End If

        'add .dic extension if file does not exist and dictionary path does not have required extension
        If Not File.Exists(filePath) AndAlso Not String.Equals(Path.GetExtension(filePath), ".dic", StringComparison.OrdinalIgnoreCase) Then
            filePath = filePath.Trim() + ".dic"
        End If

        Return filePath
    End Function

    ''' <summary>
    ''' Initializes the user dictionary.
    ''' </summary>
    ''' <param name="userDictionaryPath">The user dictionary path.</param>
    Private Sub initializeUserDictionary(userDictionaryPath As String)
        _fixedUserDictionaryPath = fixDictionaryPath(userDictionaryPath)
        If Not String.IsNullOrEmpty(_fixedUserDictionaryPath) AndAlso File.Exists(_fixedUserDictionaryPath) Then
            ' Append to user dictionary all the words from specified file
            For Each line As String In File.ReadAllLines(_fixedUserDictionaryPath)
                If Not _userDictionary.Contains(line) Then
                    _userDictionary.Add(line)
                End If
            Next
        End If
    End Sub
End Class