// ***********************************************************************
// Assembly         : WinFormHtmlEditor
// Author           : SpiceLogic
// Created          : 12-15-2012
//
// Last Modified By : SpiceLogic

// ***********************************************************************
// <copyright file="HyperLinkDialog.cs" company="SpiceLogic Inc.">
//     Copyright (c) SpiceLogic Inc.. All rights reserved.
// </copyright>
// <summary></summary>
// ***********************************************************************

using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using SpiceLogic.HtmlEditorControl.Domain.BOs.Entities;
using SpiceLogic.HtmlEditorControl.Domain.Dialogs;

namespace Dialog_Customization.Dialogs
{
    /// <summary>
    /// Class HyperLinkDialog
    /// </summary>
    public partial class HyperLinkDialog : Form, IHyperlinkDialog
    {
        /// <summary>
        /// The _the original element
        /// </summary>
        private HyperlinkElement _theOriginalElement;

        /// <summary>
        /// Whether toolTip1 is shown
        /// </summary>
        private bool _toolTipShown;

        /// <summary>
        /// Initializes a new instance of the <see cref="HyperLinkDialog" /> class.
        /// </summary>
        public HyperLinkDialog()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Handles the Load event of the HyperLinkDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void HyperLinkDialog_Load(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(this._theOriginalElement.BaseURL))
            {
                rdoWorkingDirFile.Enabled = false;
                toolTip1.SetToolTip(rdoWorkingDirFile, "You need to set Base Url in order to use this option");
            }
        }

        /// <summary>
        /// Gets or sets the element.
        /// </summary>
        /// <value>The element.</value>
        public HyperlinkElement Element
        {
            get { return readUi(); }
            set
            {
                this._theOriginalElement = value;
                this.updateUI(value);
            }
        }

        /// <summary>
        /// Reads the UI.
        /// </summary>
        /// <returns>HyperlinkElement.</returns>
        private HyperlinkElement readUi()
        {
            HyperlinkElement theElement = new HyperlinkElement
            {
                TheActiveHTMLElement = this._theOriginalElement.TheActiveHTMLElement,
                CSSStyle = this._theOriginalElement.CSSStyle,
                CSSClassName = this._theOriginalElement.CSSClassName,
                Name = this._theOriginalElement.Name,
                ID = this._theOriginalElement.ID,
                OnClickJavascript = this._theOriginalElement.OnClickJavascript,
                HrefUrl = txtURL.Text,
                Title = txtToolTip.Text.Trim(),
                InnerHtml = txtInnerHtml.Text.Trim(),
                Target = chkTargetIncluded.Checked ? cmbTarget.Text : null
            };

            return theElement;
        }

        /// <summary>
        /// Updates the UI.
        /// </summary>
        /// <param name="element">The element.</param>
        private void updateUI(HyperlinkElement element)
        {
            if (this.IsLocalResourceSelectionDisabled)
            {
                rdoLocalFile.Enabled = false;
                btnBrowseFile.Enabled = false;
            }

            txtURL.Text = element.HrefUrl;
            if (element.IsRelativePathOrUrl)
                rdoWorkingDirFile.Checked = true;
            else if (element.IsLocalFilePath && !this.IsLocalResourceSelectionDisabled)
                rdoLocalFile.Checked = true;
            txtToolTip.Text = element.Title;
            cmbTarget.Text = element.Target;
            chkTargetIncluded.Checked = (!string.IsNullOrEmpty(element.Target));
            txtInnerHtml.Text = element.InnerHtml;
         }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is local resource selection disabled.
        /// </summary>
        /// <value><c>true</c> if this instance is local resource selection disabled; otherwise, <c>false</c>.</value>
        public bool IsLocalResourceSelectionDisabled { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether [remove link].
        /// </summary>
        /// <value><c>true</c> if [remove link]; otherwise, <c>false</c>.</value>
        public bool RemoveLink
        {
            get { return chkRemoveLink.Checked; }
            set { chkRemoveLink.Checked = value; }
        }

        /// <summary>
        /// Handles the LinkClicked event of the lnkCheck control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void lnkCheck_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                Process.Start(txtURL.Text);
            }
            catch
            {
                //MessageBox.Show(EditorLangRes.ErrorURL, EditorLangRes.ErrorCallingURL);
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkTargetIncluded control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkTargetIncluded_CheckedChanged(object sender, EventArgs e)
        {
            cmbTarget.Enabled = chkTargetIncluded.Checked;
        }

        /// <summary>
        /// Handles the Click event of the btnBrowseFile control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void btnBrowseFile_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog myDialog = new OpenFileDialog())
            {
                if (myDialog.ShowDialog() == DialogResult.OK)
                    txtURL.Text = myDialog.FileName;
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the rdoLocalFile control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void rdoLocalFile_CheckedChanged(object sender, EventArgs e)
        {
            btnBrowseFile.Enabled = rdoLocalFile.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the rdoWorkingDirFile control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void rdoWorkingDirFile_CheckedChanged(object sender, EventArgs e)
        {
            bool? isLocalDir = this._theOriginalElement.IsBaseUrlALocalFolder();

            lnkImportToBaseFolder.Enabled = rdoWorkingDirFile.Enabled && rdoWorkingDirFile.Checked &&
                                            isLocalDir.HasValue && isLocalDir.Value;
            chkOverwrite.Enabled = rdoWorkingDirFile.Enabled && rdoWorkingDirFile.Checked &&
                                            isLocalDir.HasValue && isLocalDir.Value;
            lnkBrowseWD.Enabled = rdoWorkingDirFile.Enabled && rdoWorkingDirFile.Checked &&
                                            isLocalDir.HasValue && isLocalDir.Value;
        }

        /// <summary>
        /// Handles the LinkClicked event of the lnkBrowseWD control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void lnkBrowseWD_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (OpenFileDialog srcUrlDialog = new OpenFileDialog { InitialDirectory = this._theOriginalElement.BaseURL })
            {
                if (srcUrlDialog.ShowDialog() == DialogResult.OK)
                {
                    string directory = Path.GetDirectoryName(srcUrlDialog.FileName);
                    string baseUrl = this._theOriginalElement.BaseURL;
                    if (this._theOriginalElement.BaseURL.EndsWith("\\") || this._theOriginalElement.BaseURL.EndsWith("/"))
                        baseUrl = this._theOriginalElement.BaseURL.Remove(this._theOriginalElement.BaseURL.Length - 1);
                    if (baseUrl.Equals(directory, StringComparison.OrdinalIgnoreCase))
                        txtURL.Text = Path.GetFileName(srcUrlDialog.FileName);
                    else
                    {
                        DialogResult dlgResult = MessageBox.Show(@"The file you selected is not from the base directory for relative path. Do you want to import that file to your base directory ? If you choose YES, then it will be imported to the Base Directory, otherwise the link target will be treated as absolute path file.", "Selected file is not from the base directory.", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                        switch (dlgResult)
                        {
                            case DialogResult.Yes:
                                string newFilePath = Path.Combine(baseUrl, Path.GetFileName(srcUrlDialog.FileName) ?? string.Empty);
                                int i = 0;
                                while (File.Exists(newFilePath) && (!chkOverwrite.Checked))
                                {
                                    i++;
                                    string newFileName = Path.GetFileNameWithoutExtension(srcUrlDialog.FileName) + i + Path.GetExtension(srcUrlDialog.FileName);
                                    newFilePath = Path.Combine(baseUrl, newFileName);
                                }
                                if (File.Exists(newFilePath))
                                    try
                                    {
                                        File.SetAttributes(newFilePath, FileAttributes.Normal);
                                        File.Delete(newFilePath);
                                        File.Copy(srcUrlDialog.FileName, newFilePath);
                                    }
                                    catch (Exception err)
                                    {
                                        MessageBox.Show(err.Message, @"Error copying file to the destination");
                                    }
                                else
                                    File.Copy(srcUrlDialog.FileName, newFilePath);
                                txtURL.Text = Path.GetFileName(newFilePath);
                                break;
                            case DialogResult.No:
                                txtURL.Text = srcUrlDialog.FileName;
                                break;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Handles the LinkClicked event of the lnkImportToBaseFolder control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void lnkImportToBaseFolder_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (OpenFileDialog myDialog = new OpenFileDialog())
            {
                if (myDialog.ShowDialog() == DialogResult.OK)
                {
                    string newFilePath = Path.Combine(this._theOriginalElement.BaseURL, Path.GetFileName(myDialog.FileName) ?? string.Empty);
                    int i = 0;
                    while (File.Exists(newFilePath) && (!chkOverwrite.Checked))
                    {
                        i++;
                        string newFileName = Path.GetFileNameWithoutExtension(myDialog.FileName) + i + Path.GetExtension(myDialog.FileName);
                        newFilePath = Path.Combine(this._theOriginalElement.BaseURL, newFileName);
                    }
                    if (File.Exists(newFilePath))
                        try
                        {
                            File.SetAttributes(newFilePath, FileAttributes.Normal);
                            File.Delete(newFilePath);
                            File.Copy(myDialog.FileName, newFilePath);
                        }
                        catch (Exception err)
                        {
                            MessageBox.Show(err.Message, @"Error copying file to the destination");
                        }
                    else
                        File.Copy(myDialog.FileName, newFilePath);
                    txtURL.Text = Path.GetFileName(newFilePath);
                }
            }
        }

        /// <summary>
        /// Handles MouseMove event of the pnlRelativeUrl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void pnlRelativeUrl_MouseMove(object sender, MouseEventArgs e)
        {
            Control parent = sender as Control;
            if (parent == null)
            {
                return;
            }
            var ctrl = parent.GetChildAtPoint(e.Location);
            if (ctrl != null)
            {
                if (ctrl.Visible && toolTip1.Tag == null)
                {
                    if (!_toolTipShown)
                    {
                        string tipstring = toolTip1.GetToolTip(ctrl);
                        toolTip1.Show(tipstring.Trim(), ctrl, ctrl.Width / 2, ctrl.Height / 2);
                        toolTip1.Tag = ctrl;
                        _toolTipShown = true;
                    }
                }
            }
            else
            {
                ctrl = toolTip1.Tag as Control;
                if (ctrl != null)
                {
                    toolTip1.Hide(ctrl);
                    toolTip1.Tag = null;
                    _toolTipShown = false;
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the btnOK control.
        /// </summary>
        /// <param name="sender">The event source.</param>
        /// <param name="e">The event data.</param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtURL.Text))
            {
                this.DialogResult = DialogResult.None;
                MessageBox.Show("Please provide Url");
                txtURL.Focus();
            }
        }
    }
}