// ***********************************************************************

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using SpiceLogic.HtmlEditorControl.Domain.BOs.Entities;
using SpiceLogic.HtmlEditorControl.Domain.Dialogs;

namespace Dialog_Customization.Dialogs
{
    /// <summary>
    /// Class TablePropertiesDialog
    /// </summary>
    public partial class TablePropertiesDialog : Form, ITableDialog
    {
        /// <summary>
        /// The _element
        /// </summary>
        private TableElement _element;
        /// <summary>
        /// The _table cell dialog
        /// </summary>
        private readonly ITableCellDialog _tableCellDialog;
        /// <summary>
        /// The _background picture URL
        /// </summary>
        private string _backgroundPictureUrl = string.Empty;

        private readonly List<string> _propertiesAffected = new List<string>
            {
                "Rows",
                "Columns",
                "Width",
                "Height",
                "Caption",
                "BorderWidth",
                "CellPadding",
                "CellSpacing",
                "BorderColor",
                "BorderAttr",
                "BorderStyle",
                "BgColor",
                "BorderCollapse",
                "SummaryDescription",
                "ID",
                "Name",
                "CSSClassName",
                "BackGround"
            };

        /// <summary>
        /// Initializes a new instance of the <see cref="TablePropertiesDialog" /> class.
        /// </summary>
        /// <param name="cellDialog">The cell dialog.</param>
        public TablePropertiesDialog(ITableCellDialog cellDialog)
        {
            InitializeComponent();
            this.cmbBorderStyle.SelectedIndex = 0;
            this._tableCellDialog = cellDialog;
            this.CancelButton = btnCancel;
        }

        /// <summary>
        /// Handles the Load event of the TablePropertiesDialog control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void TablePropertiesDialog_Load(object sender, EventArgs e)
        {
            btnCellProperties.Visible = this.getOrInitCellElement() != null;
        }

        /// <summary>
        /// Gets or sets the element.
        /// </summary>
        /// <value>The element.</value>
        public TableElement Element
        {
            get
            {
                if (_element == null)
                    _element = new TableElement(null);

                _element.ResetValues(this._propertiesAffected);

                _element.Rows = (int)numRows.Value;
                _element.Columns = (int)numCols.Value;

                if (chkCellPadding.Checked)
                    _element.CellPadding = (int)numCellPadding.Value;
                if (chkCellSpacing.Checked)
                    _element.CellSpacing = (int)numCellSpacing.Value;

                if (chkWidth.Checked && cmbWidthUnit.SelectedItem != null)
                {
                    _element.Width = Convert.ToInt32(txtWidth.Text.Trim());
                    _element.WidthUnit = cmbWidthUnit.SelectedItem.ToString();
                }

                if (chkHeight.Checked && cmbHeightUnit.SelectedItem != null)
                {
                    _element.Height = Convert.ToInt32(txtHeight.Text.Trim());
                    _element.HeightUnit = cmbHeightUnit.SelectedItem.ToString();
                }

                if (chkCaption.Checked)
                    _element.Caption = txtCaption.Text.Trim();

                if (chkBorderWidth.Checked)
                {
                    if (chkBorderStyle.Checked && cmbBorderStyle.SelectedItem != null)
                    {
                        _element.BorderStyle = cmbBorderStyle.SelectedItem.ToString();
                        _element.BorderWidth = (int)numBorderWidth.Value;
                    }
                    else
                        _element.BorderAttr = (int)numBorderWidth.Value;
                }

                if (chklnkBorderColor.Enabled && chklnkBorderColor.Checked)
                    _element.BorderColor = ColorTranslator.ToHtml(txtBorderColor.BackColor);

                if (chklnkBgColor.Checked)
                    _element.BgColor = ColorTranslator.ToHtml(txtBgColor.BackColor);

                _element.SummaryDescription = txtSummaryDescription.Text;
                _element.BorderCollapse = chkBorderCollapse.Enabled && chkBorderCollapse.Checked;
                _element.ID = txtId.Text.Trim();
                _element.Name = txtName.Text.Trim();
                _element.CSSClassName = txtClassName.Text.Trim();
                _element.CSSStyle = txtCss.Text.Trim();

                if (chklnkBackgroundPicture.Checked)
                    _element.BackGround = pbBackgroundPicture.ImageLocation;

                _element.BorderToAll = chkBorderToAll.Checked;

                return _element;
            }
            set
            {
                _element = value;
                if (_element == null)
                    return;

                numRows.Value = _element.Rows;
                numCols.Value = _element.Columns;

                chkWidth.Checked = _element.Width.HasValue;

                if (chkWidth.Checked)
                {
                    txtWidth.Text = _element.Width.HasValue ? _element.Width.Value.ToString(CultureInfo.InvariantCulture) : "";
                    cmbWidthUnit.SelectedItem = _element.WidthUnit;
                }

                chkHeight.Checked = _element.Height.HasValue;

                if (chkHeight.Checked)
                {
                    txtHeight.Text = _element.Height.HasValue ? _element.Height.Value.ToString(CultureInfo.InvariantCulture) : "";
                    cmbHeightUnit.SelectedItem = _element.HeightUnit;
                }

                chkCaption.Checked = !string.IsNullOrEmpty(_element.Caption);

                if (chkCaption.Checked)
                    txtCaption.Text = _element.Caption;

                chkBorderWidth.Checked = _element.BorderWidth.HasValue || _element.BorderAttr.HasValue;

                chkBorderStyle.Checked = !string.IsNullOrEmpty(_element.BorderStyle);
                cmbBorderStyle.Enabled = chkBorderStyle.Checked;
                chkBorderCollapse.Enabled = chkBorderStyle.Checked;
                chklnkBorderColor.Enabled = chkBorderStyle.Checked;
                this.txtBorderColor.Enabled = this.chkBorderStyle.Checked && this.chklnkBorderColor.Checked;

                if (chkBorderStyle.Checked)
                {
                    cmbBorderStyle.Text = _element.BorderStyle;
                    if (chkBorderWidth.Checked)
                        numBorderWidth.Value = _element.BorderWidth ?? 0;
                }
                else
                {
                    if (chkBorderWidth.Checked)
                        numBorderWidth.Value = _element.BorderAttr ?? (_element.BorderWidth ?? 0);
                }

                chkCellPadding.Checked = _element.CellPadding.HasValue;

                if (chkCellPadding.Checked)
                    numCellPadding.Value = _element.CellPadding ?? 0;

                chkCellSpacing.Checked = _element.CellSpacing.HasValue;
                if (chkCellSpacing.Checked)
                {
                    if (_element.CellSpacing != null)
                        numCellSpacing.Value = _element.CellSpacing.Value;
                    chkCellSpacing.Checked = true;
                }

                chklnkBorderColor.Checked = !string.IsNullOrEmpty(_element.BorderColor);
                if (chklnkBorderColor.Checked)
                {
                    ColorConverter converter = new ColorConverter();
                    txtBorderColor.BackColor = (Color)converter.ConvertFromString(_element.BorderColor);
                }

                chklnkBgColor.Checked = !string.IsNullOrEmpty(_element.BgColor);

                if (chklnkBgColor.Checked)
                {
                    ColorConverter converter = new ColorConverter();
                    txtBgColor.BackColor = (Color)converter.ConvertFromString(_element.BgColor);
                }

                chkBorderCollapse.Checked = _element.BorderCollapse != null;
                if (chkBorderCollapse.Checked)
                    chkBorderCollapse.Checked = _element.BorderCollapse.Value;
                txtSummaryDescription.Text = _element.SummaryDescription;
                txtId.Text = _element.ID;
                txtName.Text = _element.Name;
                txtClassName.Text = _element.CSSClassName;
                /////////////////
                chklnkBackgroundPicture.Checked = !string.IsNullOrEmpty(_element.BackGround);

                if (chklnkBackgroundPicture.Checked)
                {
                    pbBackgroundPicture.ImageLocation = _element.BackGround;
                    pbBackgroundPicture.Load();
                }

                txtCss.Text = _element.GetCssStyleWithoutProperties(this._propertiesAffected);
                chkBorderToAll.Checked = true;
            }
        }

        #region ------------ UI Event Handlers --------------

        /// <summary>
        /// Handles the CheckedChanged event of the chkWidth control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkWidth_CheckedChanged(object sender, EventArgs e)
        {
            txtWidth.Enabled = chkWidth.Checked;
            cmbWidthUnit.Enabled = chkWidth.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkHeight control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkHeight_CheckedChanged(object sender, EventArgs e)
        {
            txtHeight.Enabled = chkHeight.Checked;
            cmbHeightUnit.Enabled = chkHeight.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkBorderWidth control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkBorderWidth_CheckedChanged(object sender, EventArgs e)
        {
            numBorderWidth.Enabled = chkBorderWidth.Checked;

            this.chkBorderStyle.Enabled = this.chkBorderWidth.Checked;

            //            chkBorderStyle.Checked = chkBorderWidth.Checked;
            cmbBorderStyle.Enabled = chkBorderWidth.Checked && chkBorderStyle.Checked;
            if (!chkBorderWidth.Checked) chkBorderToAll.Checked = true;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkCellPadding control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkCellPadding_CheckedChanged(object sender, EventArgs e)
        {
            numCellPadding.Enabled = chkCellPadding.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkCellSpacing control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkCellSpacing_CheckedChanged(object sender, EventArgs e)
        {
            numCellSpacing.Enabled = chkCellSpacing.Checked;
        }

        /// <summary>
        /// Handles the LinkClicked event of the chklnkBorderColor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void chklnkBorderColor_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (ColorDialog myDialog = new ColorDialog
            {
                AllowFullOpen = true,
                AnyColor = true,
                Color = txtBorderColor.BackColor,
                CustomColors = new[] { ColorTranslator.ToOle(txtBorderColor.BackColor) }
            })

            if (myDialog.ShowDialog() == DialogResult.OK)
                txtBorderColor.BackColor = myDialog.Color;
        }

        /// <summary>
        /// Handles the LinkClicked event of the chklnkBgColor control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void chklnkBgColor_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (ColorDialog myDialog = new ColorDialog
            {
                AllowFullOpen = true,
                AnyColor = true,
                Color = txtBgColor.BackColor,
                CustomColors = new[] { ColorTranslator.ToOle(txtBgColor.BackColor) }
            })

            if (myDialog.ShowDialog() == DialogResult.OK)
                    txtBgColor.BackColor = myDialog.Color;
        }

        /// <summary>
        /// Handles the LinkClicked event of the chklnkBackgroundPicture control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.LinkLabelLinkClickedEventArgs" /> instance containing the event data.</param>
        private void chklnkBackgroundPicture_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try
            {
                using (OpenFileDialog srcUrlDialog = new OpenFileDialog { Title = @"Please Select an image file.", RestoreDirectory = true, Filter = @"Image Files|*.png;*.bmp;*.gif;*.jpg|All files(*.*)|*.*", FilterIndex = 0, Multiselect = false })
                {
                    if (srcUrlDialog.ShowDialog() == DialogResult.OK)
                    {
                        _backgroundPictureUrl = srcUrlDialog.FileName;
                        pbBackgroundPicture.Image = Image.FromFile(_backgroundPictureUrl);
                        pbBackgroundPicture.ImageLocation = _backgroundPictureUrl;
                    }
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public new DialogResult ShowDialog()
        {
            this._tableCellDialog.Element = null;
            return base.ShowDialog();
        }

        /// <summary>
        /// Handles the Click event of the btnCellProperties control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void btnCellProperties_Click(object sender, EventArgs e)
        {
            if (this._tableCellDialog == null)
            {
                MessageBox.Show(
                    "Table Cell Dialog reference is NULL. Please set a TableCellDialog implementing ITableCellDialog to the editor's Dialog.TableCellDialog property.",
                    "Error");
                return;
            }

            TableCellElement tblCellElement = this.getOrInitCellElement();
            if (tblCellElement == null)
            {
                MessageBox.Show(
                    "No cells were found.",
                    "Error");
                return;
            }

            this._tableCellDialog.Element = tblCellElement;
            using (this._tableCellDialog.LockOverrideSettingsToAllCells())
            {
                if (this._tableCellDialog.ShowDialog() == DialogResult.OK)
                    this.Element.CellElement = this._tableCellDialog.Element;
            }
        }

        private TableCellElement getOrInitCellElement()
        {
            return this.Element.GetFirstCellElement();
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkCaption control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkCaption_CheckedChanged(object sender, EventArgs e)
        {
            txtCaption.Enabled = chkCaption.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chkBorderStyle control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        private void chkBorderStyle_CheckedChanged(object sender, EventArgs e)
        {
            cmbBorderStyle.Enabled = chkBorderStyle.Checked;
            chkBorderCollapse.Enabled = chkBorderStyle.Checked;
            chklnkBorderColor.Enabled = chkBorderStyle.Checked;
            this.txtBorderColor.Enabled = this.chkBorderStyle.Checked && this.chklnkBorderColor.Checked;
            if (!chkBorderStyle.Checked) chkBorderToAll.Checked = true;
        }

        private void numBorderWidth_ValueChanged(object sender, EventArgs e)
        {
            if (this.numBorderWidth.Value == 0.0m)
                this.chkBorderToAll.Checked = true;
        }

        #endregion
    }
}